@@ -21,13 +21,14 @@
 #ifndef _OSIP_H_
 #define _OSIP_H_
 
+#if defined(HAVE_DICT_DICT_H)
+#include <dict/dict.h>
+#endif
+
 #include <osipparser2/osip_const.h>
 
-#include <time.h>
-
-#ifdef WIN32
-#include <Winsock.h>
-#endif
+/* Time-related functions and data types */
+#include <osip2/osip_time.h>
 
 #ifdef __sun
 #include <sys/types.h>
@@ -92,7 +93,7 @@ extern "C"
 #ifndef DOXYGEN
     DIALOG_EARLY,
     DIALOG_CONFIRMED,
-    DIALOG_CLOSE		/* ?? */
+    DIALOG_CLOSE                /* ?? */
 #endif
   }
   state_t;
@@ -117,26 +118,26 @@ extern "C"
   typedef enum type_t
   {
     /* TIMEOUT EVENTS for ICT */
-    TIMEOUT_A,	 /**< Timer A */
-    TIMEOUT_B,	 /**< Timer B */
-    TIMEOUT_D,	 /**< Timer D */
+    TIMEOUT_A,   /**< Timer A */
+    TIMEOUT_B,   /**< Timer B */
+    TIMEOUT_D,   /**< Timer D */
 
     /* TIMEOUT EVENTS for NICT */
-    TIMEOUT_E,	 /**< Timer E */
-    TIMEOUT_F,	 /**< Timer F */
-    TIMEOUT_K,	 /**< Timer K */
+    TIMEOUT_E,   /**< Timer E */
+    TIMEOUT_F,   /**< Timer F */
+    TIMEOUT_K,   /**< Timer K */
 
     /* TIMEOUT EVENTS for IST */
-    TIMEOUT_G,	 /**< Timer G */
-    TIMEOUT_H,	 /**< Timer H */
-    TIMEOUT_I,	 /**< Timer I */
+    TIMEOUT_G,   /**< Timer G */
+    TIMEOUT_H,   /**< Timer H */
+    TIMEOUT_I,   /**< Timer I */
 
     /* TIMEOUT EVENTS for NIST */
-    TIMEOUT_J,	 /**< Timer J */
+    TIMEOUT_J,   /**< Timer J */
 
     /* FOR INCOMING MESSAGE */
     RCV_REQINVITE,    /**< Event is an incoming INVITE request */
-    RCV_REQACK,	      /**< Event is an incoming ACK request */
+    RCV_REQACK,       /**< Event is an incoming ACK request */
     RCV_REQUEST,      /**< Event is an incoming NON-INVITE and NON-ACK request */
     RCV_STATUS_1XX,   /**< Event is an incoming informational response */
     RCV_STATUS_2XX,   /**< Event is an incoming 2XX response */
@@ -144,7 +145,7 @@ extern "C"
 
     /* FOR OUTGOING MESSAGE */
     SND_REQINVITE,    /**< Event is an outgoing INVITE request */
-    SND_REQACK,	      /**< Event is an outgoing ACK request */
+    SND_REQACK,       /**< Event is an outgoing ACK request */
     SND_REQUEST,      /**< Event is an outgoing NON-INVITE and NON-ACK request */
     SND_STATUS_1XX,   /**< Event is an outgoing informational response */
     SND_STATUS_2XX,   /**< Event is an outgoing 2XX response */
@@ -177,21 +178,21 @@ extern "C"
  * You can re-define the default value for T1. (T1 is defined in rfcxxxx)
  * The default value is 500ms.
  */
-#define DEFAULT_T1 500		/* 500 ms */
+#define DEFAULT_T1 500          /* 500 ms */
 #endif
 #ifndef DEFAULT_T2
 /**
  * You can re-define the default value for T2. (T2 is defined in rfcxxxx)
  * The default value is 4000ms.
  */
-#define DEFAULT_T2 4000		/* 4s */
+#define DEFAULT_T2 4000         /* 4s */
 #endif
 #ifndef DEFAULT_T4
 /**
  * You can re-define the default value for T4. (T1 is defined in rfcxxxx)
  * The default value is 5000ms.
  */
-#define DEFAULT_T4 5000		/* 5s */
+#define DEFAULT_T4 5000         /* 5s */
 #endif
 
 
@@ -207,13 +208,13 @@ extern "C"
  */
   struct osip_ict
   {
-    int timer_a_length;		  /**@internal A=T1, A=2xT1... (unreliable only) */
+    int timer_a_length;           /**@internal A=T1, A=2xT1... (unreliable only) */
     struct timeval timer_a_start; /**@internal */
-    int timer_b_length;		  /**@internal B = 64* T1 */
+    int timer_b_length;           /**@internal B = 64* T1 */
     struct timeval timer_b_start; /**@internal fire when transaction timeouts */
-    int timer_d_length;		  /**@internal D >= 32s for unreliable tr (or 0) */
+    int timer_d_length;           /**@internal D >= 32s for unreliable tr (or 0) */
     struct timeval timer_d_start; /**@internal should be equal to timer H */
-    char *destination;	          /**@internal url used to send requests */
+    char *destination;            /**@internal url used to send requests */
     int port;                     /**@internal port of next hop */
   };
 
@@ -229,14 +230,14 @@ extern "C"
  */
   struct osip_nict
   {
-    int timer_e_length;		  /**@internal A=T1, A=2xT1... (unreliable only) */
+    int timer_e_length;           /**@internal A=T1, A=2xT1... (unreliable only) */
     struct timeval timer_e_start; /**@internal */
-    int timer_f_length;		  /**@internal B = 64* T1 */
+    int timer_f_length;           /**@internal B = 64* T1 */
     struct timeval timer_f_start; /**@internal fire when transaction timeouts */
-    int timer_k_length;		  /**@internal K = T4 (else = 0) */
+    int timer_k_length;           /**@internal K = T4 (else = 0) */
     struct timeval timer_k_start; /**@internal */
-    char *destination;		  /**@internal url used to send requests */
-    int port;			  /**@internal port of next hop */
+    char *destination;            /**@internal url used to send requests */
+    int port;                     /**@internal port of next hop */
 
   };
 
@@ -252,11 +253,11 @@ extern "C"
  */
   struct osip_ist
   {
-    int timer_g_length;	     /**@internal G=MIN(T1*2,T2) for unreliable trans. */
+    int timer_g_length;      /**@internal G=MIN(T1*2,T2) for unreliable trans. */
     struct timeval timer_g_start; /**@internal 0 when reliable transport is used */
-    int timer_h_length;		  /**@internal H = 64* T1 */
+    int timer_h_length;           /**@internal H = 64* T1 */
     struct timeval timer_h_start; /**@internal fire when if no ACK is received */
-    int timer_i_length;		  /**@internal I = T4 for unreliable (or 0) */
+    int timer_i_length;           /**@internal I = T4 for unreliable (or 0) */
     struct timeval timer_i_start; /**@internal absorb all ACK */
   };
 
@@ -272,7 +273,7 @@ extern "C"
  */
   struct osip_nist
   {
-    int timer_j_length;		   /**@internal J = 64*T1 (else 0) */
+    int timer_j_length;            /**@internal J = 64*T1 (else 0) */
     struct timeval timer_j_start;  /**@internal */
   };
 
@@ -289,29 +290,29 @@ extern "C"
   struct osip_transaction
   {
 
-    void *your_instance;	/**< User Defined Pointer. */
-    int transactionid;		/**< Internal Transaction Identifier. */
-    osip_fifo_t *transactionff;	/**< events must be added in this fifo */
+    void *your_instance;        /**< User Defined Pointer. */
+    int transactionid;          /**< Internal Transaction Identifier. */
+    osip_fifo_t *transactionff; /**< events must be added in this fifo */
 
-    osip_via_t *topvia;		/**< CALL-LEG definition (Top Via) */
-    osip_from_t *from;		/**< CALL-LEG definition (From)    */
+    osip_via_t *topvia;         /**< CALL-LEG definition (Top Via) */
+    osip_from_t *from;          /**< CALL-LEG definition (From)    */
     osip_to_t *to;              /**< CALL-LEG definition (To)      */
     osip_call_id_t *callid;     /**< CALL-LEG definition (Call-ID) */
     osip_cseq_t *cseq;          /**< CALL-LEG definition (CSeq)    */
 
     osip_message_t *orig_request;  /**< Initial request            */
     osip_message_t *last_response; /**< Last response              */
-    osip_message_t *ack;	   /**< ack request sent           */
+    osip_message_t *ack;           /**< ack request sent           */
 
-    state_t state;		/**< Current state of the transaction */
+    state_t state;              /**< Current state of the transaction */
 
-    time_t birth_time;		/**< birth date of transaction        */
-    time_t completed_time;	/**< end   date of transaction        */
+    time_t birth_time;          /**< birth date of transaction        */
+    time_t completed_time;      /**< end   date of transaction        */
 
     int in_socket;              /**< Optional socket for incoming message */
     int out_socket;             /**< Optional place for outgoing message */
 
-    void *config;		/**@internal transaction is managed by osip_t  */
+    void *config;               /**@internal transaction is managed by osip_t  */
 
     osip_fsm_type_t ctx_type;   /**< Type of the transaction */
     osip_ict_t *ict_context;    /**@internal */
@@ -389,6 +390,9 @@ extern "C"
     OSIP_NIST_STATUS_6XX_SENT,              /**< 6XX FOR MESSAGE SENT */
     OSIP_NIST_STATUS_3456XX_SENT_AGAIN,     /**< RESPONSE RETRANSMITTED */
 
+    OSIP_ICT_STATUS_TIMEOUT,                /**< TIMER B EXPIRATION: NO REMOTE ANSWER  */
+    OSIP_NICT_STATUS_TIMEOUT,               /**< TIMER F EXPIRATION: NO REMOTE ANSWER  */
+
     OSIP_MESSAGE_CALLBACK_COUNT             /**< END OF ENUM */
   } osip_message_callback_type_t;
 
@@ -424,7 +428,7 @@ extern "C"
  * @var osip_message_cb_t
  */
   typedef void (*osip_message_cb_t) (int type, osip_transaction_t *,
-				     osip_message_t *);
+                                     osip_message_t *);
 /**
  * Callback definition for end of transaction announcements.
  * @var osip_kill_transaction_cb_t
@@ -435,7 +439,7 @@ extern "C"
  * @var osip_transport_error_cb_t
  */
   typedef void (*osip_transport_error_cb_t) (int type, osip_transaction_t *,
-					     int error);
+                                             int error);
 
 
   struct osip_dialog;
@@ -461,7 +465,7 @@ extern "C"
     char *dest;                 /**< destination host */
     int port;                   /**< destination port */
     int sock;                   /**< socket to use */
-    int counter;		/**< start at 7 */
+    int counter;                /**< start at 7 */
   };
 
 
@@ -481,22 +485,30 @@ extern "C"
   struct osip
   {
 
-    void *application_context;	/**< User defined Pointer */
+    void *application_context;  /**< User defined Pointer */
 
     /* list of transactions for ict, ist, nict, nist */
-    osip_list_t *osip_ict_transactions;  /**< list of ict transactions */
-    osip_list_t *osip_ist_transactions;  /**< list of ist transactions */
-    osip_list_t *osip_nict_transactions; /**< list of nict transactions */
-    osip_list_t *osip_nist_transactions; /**< list of nist transactions */
+    osip_list_t *osip_ict_transactions;   /**< list of ict transactions */
+    osip_list_t *osip_ist_transactions;   /**< list of ist transactions */
+    osip_list_t *osip_nict_transactions;  /**< list of nict transactions */
+    osip_list_t *osip_nist_transactions;  /**< list of nist transactions */
 
-    osip_list_t *ixt_retransmissions;	 /**< list of ixt elements */
+    osip_list_t *ixt_retransmissions;    /**< list of ixt elements */
 
     osip_message_cb_t msg_callbacks[OSIP_MESSAGE_CALLBACK_COUNT];        /**@internal */
     osip_kill_transaction_cb_t kill_callbacks[OSIP_KILL_CALLBACK_COUNT]; /**@internal */
     osip_transport_error_cb_t tp_error_callbacks[OSIP_TRANSPORT_ERROR_CALLBACK_COUNT];  /**@internal */
 
     int (*cb_send_message) (osip_transaction_t *, osip_message_t *, char *,
-			    int, int);  /**@internal */
+                            int, int);
+                                        /**@internal */
+
+#if defined(HAVE_DICT_DICT_H)
+    dict *osip_ict_hastable;              /**< htable of ict transactions */
+    dict *osip_ist_hastable;              /**< htable of ist transactions */
+    dict *osip_nict_hastable;             /**< htable of nict transactions */
+    dict *osip_nist_hastable;             /**< htable of nist transactions */
+#endif
   };
 
 /**
@@ -505,7 +517,7 @@ extern "C"
  * @param type The event type to hook on.
  * @param cb The method to be called upon the event.
  */
-  int osip_set_message_callback (osip_t *osip, int type, osip_message_cb_t cb);
+  int osip_set_message_callback (osip_t * osip, int type, osip_message_cb_t cb);
 
 /**
  * Set a callback for transaction operation related to the end of transactions. 
@@ -513,8 +525,8 @@ extern "C"
  * @param type The event type to hook on.
  * @param cb The method to be called upon the event.
  */
-  int osip_set_kill_transaction_callback (osip_t *osip, int type,
-					  osip_kill_transaction_cb_t cb);
+  int osip_set_kill_transaction_callback (osip_t * osip, int type,
+                                          osip_kill_transaction_cb_t cb);
 
 /**
  * Set a callback for each transaction operation related to network error.
@@ -522,8 +534,8 @@ extern "C"
  * @param type The event type to hook on.
  * @param cb The method to be called upon the event.
  */
-  int osip_set_transport_error_callback (osip_t *osip, int type,
-					 osip_transport_error_cb_t cb);
+  int osip_set_transport_error_callback (osip_t * osip, int type,
+                                         osip_transport_error_cb_t cb);
 
 /**
  * Structure for osip event handling.
@@ -556,8 +568,8 @@ extern "C"
  * @param request The SIP request that initiate the transaction.
  */
   int osip_transaction_init (osip_transaction_t ** transaction,
-			     osip_fsm_type_t ctx_type, osip_t * osip,
-			     osip_message_t * request);
+                             osip_fsm_type_t ctx_type, osip_t * osip,
+                             osip_message_t * request);
 /**
  * Free all resource in a osip_transaction_t element.
  * @param transaction The element to free.
@@ -580,8 +592,7 @@ extern "C"
  * @param portnum a pointer to receive the host port.
  */
   void osip_response_get_destination (osip_message_t * response,
-				      char **address,
-				      int *portnum);
+                                      char **address, int *portnum);
 /**
  * Set the host and port destination used for sending the SIP message.
  * This can be useful for an application with 'DIRECT ROOTING MODE'
@@ -591,8 +602,7 @@ extern "C"
  * @param destination The destination host.
  * @param port The destination port.
  */
-  int osip_ict_set_destination (osip_ict_t * ict, char *destination,
-				int port);
+  int osip_ict_set_destination (osip_ict_t * ict, char *destination, int port);
 
 /**
  * Set the host and port destination used for sending the SIP message.
@@ -603,8 +613,7 @@ extern "C"
  * @param destination The destination host.
  * @param port The destination port.
  */
-  int osip_nict_set_destination (osip_nict_t * nict, char *destination,
-				 int port);
+  int osip_nict_set_destination (osip_nict_t * nict, char *destination, int port);
 
 /**
  * Add a SIP event in the fifo of a osip_transaction_t element.
@@ -612,7 +621,7 @@ extern "C"
  * @param evt The event to add.
  */
   int osip_transaction_add_event (osip_transaction_t * transaction,
-				  osip_event_t * evt);
+                                  osip_event_t * evt);
 /**
  * Consume one osip_event_t element previously added in the fifo.
  * NOTE: This method MUST NEVER be called within another call
@@ -622,7 +631,7 @@ extern "C"
  * @param evt The element to consume.
  */
   int osip_transaction_execute (osip_transaction_t * transaction,
-				osip_event_t * evt);
+                                osip_event_t * evt);
 /**
  * Set a pointer to your personal context associated with this transaction.
  * NOTE: this is a very useful method that allow you to avoid searching
@@ -635,7 +644,7 @@ extern "C"
  * @param instance The address of your context.
  */
   int osip_transaction_set_your_instance (osip_transaction_t * transaction,
-					  void *instance);
+                                          void *instance);
 /**
  * Get a pointer to your personal context associated with this transaction.
  * @param transaction The element to work on.
@@ -650,9 +659,8 @@ extern "C"
  * @param port The port where to send initial request.
  */
   int osip_transaction_get_destination (osip_transaction_t * transaction,
-					char **ip, int *port);
+                                        char **ip, int *port);
 
-#ifndef DOXYGEN
 
 /**
  * Set the socket for incoming message.
@@ -660,36 +668,16 @@ extern "C"
  * @param transaction The element to work on.
  * @param sock The socket for incoming message.
  */
-  int osip_transaction_set_in_socket (osip_transaction_t * transaction,
-				      int sock);
+  int osip_transaction_set_in_socket (osip_transaction_t * transaction, int sock);
 /**
  * Set the socket for outgoing message.
  * NOTE: THIS HAS NEVER TESTED! Please send feedback.
  * @param transaction The element to work on.
  * @param sock The socket for outgoing message.
  */
-  int osip_transaction_set_out_socket (osip_transaction_t * transaction,
-				       int sock);
+  int osip_transaction_set_out_socket (osip_transaction_t * transaction, int sock);
 
 
-#if 0
-
-
-/**
- * Check if the first 2 parameters match the other ones.
- * NOTE: THIS IS AN INTERNAL METHOD ONLY
- * @param to1 The initial to header.
- * @param from1 The initial from header.
- * @param to2 The new to header.
- * @param from2 The new from header.
- */
-  int callleg_match (osip_to_t * to1, osip_from_t * from1, osip_to_t * to2,
-		     osip_from_t * from2);
-
-#endif
-
-#endif				/* endif DOXYGEN */
-
 
 /** 
  * Allocate an osip_t element.
@@ -785,7 +773,7 @@ extern "C"
  * @param evt The element representing the SIP MESSAGE.
  */
   osip_transaction_t *osip_transaction_find (osip_list_t * transactions,
-					     osip_event_t * evt);
+                                             osip_event_t * evt);
 
 
 #ifndef DOXYGEN
@@ -797,13 +785,11 @@ extern "C"
  * @param evt The element representing the SIP MESSAGE.
  */
 #ifndef OSIP_MT
-  osip_transaction_t *osip_find_transaction (osip_t * osip,
-					     osip_event_t * evt);
+  osip_transaction_t *osip_find_transaction (osip_t * osip, osip_event_t * evt);
 #endif
 
   osip_transaction_t *__osip_find_transaction (osip_t * osip,
-					       osip_event_t * evt,
-					       int consume);
+                                               osip_event_t * evt, int consume);
 #endif
 
 /**
@@ -819,8 +805,7 @@ extern "C"
  * @param osip The element to work on.
  * @param evt The element representing the new SIP REQUEST.
  */
-  osip_transaction_t *osip_create_transaction (osip_t * osip,
-					       osip_event_t * evt);
+  osip_transaction_t *osip_create_transaction (osip_t * osip, osip_event_t * evt);
 
 /**
  * Create a sipevent from a SIP message string.
@@ -844,8 +829,8 @@ extern "C"
  * @param sock The socket to be used to send the message. (optional).
  */
   void osip_start_200ok_retransmissions (osip_t * osip,
-					 struct osip_dialog *dialog,
-					 osip_message_t * msg200ok, int sock);
+                                         struct osip_dialog *dialog,
+                                         osip_message_t * msg200ok, int sock);
 
 /**
  * Start out of fsm ACK retransmissions. This is usefull for user-agents.
@@ -853,20 +838,21 @@ extern "C"
  * @param dialog The dialog the ACK is part of.
  * @param ack The ACK that has just been sent in response to a 200 Ok.
  * @param dest The destination host.
- * @param sock The destination port.
+ * @param port The destination port.
  * @param sock The socket to be used to send the message. (optional).
  */
   void osip_start_ack_retransmissions (osip_t * osip,
-				       struct osip_dialog *dialog,
-				       osip_message_t * ack, char *dest,
-				       int port, int sock);
+                                       struct osip_dialog *dialog,
+                                       osip_message_t * ack, char *dest,
+                                       int port, int sock);
 
 /**
  * Stop the out of fsm 200 Ok retransmissions matching an incoming ACK.
  * @param osip The osip_t structure.
  * @param ack  The ack that has just been received.
  */
-  struct osip_dialog *osip_stop_200ok_retransmissions (osip_t * osip, osip_message_t * ack);
+  struct osip_dialog *osip_stop_200ok_retransmissions (osip_t * osip,
+                                                       osip_message_t * ack);
 
 /**
  * Stop out of fsm retransmissions (ACK or 200 Ok) associated to a given dialog.
@@ -876,7 +862,7 @@ extern "C"
  * @param dialog The dialog.
  */
   void osip_stop_retransmissions_from_dialog (osip_t * osip,
-					      struct osip_dialog *dialog);
+                                              struct osip_dialog *dialog);
 
 
 /**
@@ -897,9 +883,8 @@ extern "C"
  * @param cb The method we want to register.
  */
   void osip_set_cb_send_message (osip_t * cf,
-				 int (*cb) (osip_transaction_t *,
-					    osip_message_t *, char *,
-					    int, int));
+                                 int (*cb) (osip_transaction_t *,
+                                            osip_message_t *, char *, int, int));
 
 /* FOR INCOMING TRANSACTION */
 /**
